/* eslint-disable */
/* See CanvasSectionContainer.ts for explanations. */
var ScrollSection = /** @class */ (function () {
    function ScrollSection() {
        this.context = null;
        this.myTopLeft = null;
        this.documentTopLeft = null;
        this.containerObject = null;
        this.dpiScale = null;
        this.name = null;
        this.backgroundColor = null;
        this.borderColor = null;
        this.boundToSection = L.CSections.Tiles.name;
        this.anchor = new Array(0);
        this.position = new Array(0);
        this.size = new Array(0);
        this.expand = new Array(0);
        this.isLocated = false;
        this.processingOrder = null;
        this.drawingOrder = null;
        this.zIndex = null;
        this.interactable = true;
        this.isAnimating = false; // This variable is set by the CanvasSectionContainer class.
        this.sectionProperties = {};
        this.name = L.CSections.Scroll.name;
        this.anchor = ['top', 'right'];
        this.position = [0, 0];
        this.size = [30 * window.devicePixelRatio, 0];
        this.expand = ['bottom'];
        this.processingOrder = L.CSections.Scroll.processingOrder;
        this.drawingOrder = L.CSections.Scroll.drawingOrder;
        this.zIndex = L.CSections.Scroll.zIndex;
        this.map = L.Map.THIS;
        this.map.on('scrollto', this.onScrollTo, this);
        this.map.on('scrollby', this.onScrollBy, this);
        this.map.on('scrollvelocity', this.onScrollVelocity, this);
        this.map.on('handleautoscroll', this.onHandleAutoScroll, this);
        this.map.on('docsize', this.onUpdateSize, this);
        this.map.on('updatescrolloffset', this.onUpdateScrollOffset, this);
    }
    ScrollSection.prototype.onInitialize = function () {
        this.sectionProperties.roundedDpi = Math.round(this.dpiScale);
        if (this.sectionProperties.roundedDpi === 0)
            this.sectionProperties.roundedDpi = 1;
        this.sectionProperties.mapPane = (document.querySelectorAll('.leaflet-map-pane')[0]);
        this.sectionProperties.defaultCursorStyle = this.sectionProperties.mapPane.style.cursor;
        this.sectionProperties.documentWidth = 0;
        this.sectionProperties.documentHeight = 0;
        this.sectionProperties.documentTopMax = Infinity;
        this.sectionProperties.documentRightMax = Infinity;
        this.sectionProperties.previousDragDistance = null;
        this.sectionProperties.usableThickness = 20 * this.sectionProperties.roundedDpi;
        this.sectionProperties.scrollBarThickness = 6 * this.sectionProperties.roundedDpi;
        this.sectionProperties.edgeOffset = 10 * this.sectionProperties.roundedDpi;
        this.sectionProperties.drawVerticalScrollBar = false;
        this.sectionProperties.drawHorizontalScrollBar = false;
        this.sectionProperties.clickScrollVertical = false; // true when user presses on the scroll bar drawing.
        this.sectionProperties.clickScrollHorizontal = false;
        this.sectionProperties.mouseIsOnVerticalScrollBar = false;
        this.sectionProperties.mouseIsOnHorizontalScrollBar = false;
        this.sectionProperties.minimumScrollSize = 80 * this.sectionProperties.roundedDpi;
        this.sectionProperties.circleSliderRadius = 24 * this.sectionProperties.roundedDpi; // Radius of the mobile vertical circular slider.
        this.sectionProperties.arrowCornerLength = 10 * this.sectionProperties.roundedDpi; // Corner length of the arrows inside circular slider.
        // Opacity.
        this.sectionProperties.alphaWhenVisible = 0.5; // Scroll bar is visible but not being used.
        this.sectionProperties.alphaWhenBeingUsed = 0.8; // Scroll bar is being used.
        this.sectionProperties.currentAlpha = 1.0; // This variable will be updated while animating. When not animating, this will be equal to one of the above variables.
        this.sectionProperties.yOffset = 0;
        this.sectionProperties.xOffset = 0;
        this.sectionProperties.horizontalScrollRightOffset = this.sectionProperties.usableThickness * 2; // To prevent overlapping of the scroll bars.
        this.sectionProperties.animatingVerticalScrollBar = false;
        this.sectionProperties.animatingHorizontalScrollBar = false;
    };
    ScrollSection.prototype.onScrollTo = function (e) {
        // Triggered by the document (e.g. search result out of the viewing area).
        this.map.scrollTop(e.y, {});
        this.map.scrollLeft(e.x, {});
    };
    ScrollSection.prototype.onScrollBy = function (e) {
        e.y *= (-1);
        var y = '+=' + e.y;
        if (e.y < 0) {
            y = '-=' + Math.abs(e.y);
        }
        e.x *= (-1);
        var x = '+=' + e.x;
        if (e.x < 0) {
            x = '-=' + Math.abs(e.x);
        }
        this.onScrollTo({ x: x, y: y });
    };
    ScrollSection.prototype.onScrollVelocity = function (e) {
        if (e.vx === 0 && e.vy === 0) {
            clearInterval(this.autoScrollTimer);
            this.autoScrollTimer = null;
            this.map.isAutoScrolling = false;
        }
        else {
            clearInterval(this.autoScrollTimer);
            this.map.isAutoScrolling = true;
            this.autoScrollTimer = setInterval(L.bind(function () {
                this.onScrollBy({ x: e.vx, y: e.vy });
            }, this), 100);
        }
    };
    ScrollSection.prototype.onHandleAutoScroll = function (e) {
        var vx = 0;
        var vy = 0;
        if (e.pos.y > e.map._size.y - 50) {
            vy = 50;
        }
        else if (e.pos.y < 50) {
            vy = -50;
        }
        if (e.pos.x > e.map._size.x - 50) {
            vx = 50;
        }
        else if (e.pos.x < 50) {
            vx = -50;
        }
        this.onScrollVelocity({ vx: vx, vy: vy });
    };
    ScrollSection.prototype.onUpdateSize = function (e) {
        // we need to avoid precision issues in comparison (in the end values are pixels)
        var newDocWidth = Math.ceil(e.x);
        var newDocHeight = Math.ceil(e.y);
        // Don't get them through L.DomUtil.getStyle because precision is no more than 6 digits
        this.sectionProperties.documentWidth = Math.round(newDocWidth * this.dpiScale);
        this.sectionProperties.documentHeight = Math.round(newDocHeight * this.dpiScale);
    };
    ScrollSection.prototype.getVerticalScrollLength = function () {
        if (this.map._docLayer._docType !== 'spreadsheet') {
            return this.size[1];
        }
        else {
            var splitPanesContext = this.map.getSplitPanesContext();
            var splitPos = splitPanesContext.getSplitPos().clone();
            splitPos.y = Math.round(splitPos.y * this.dpiScale);
            this.sectionProperties.yOffset = splitPos.y;
            return this.size[1] - splitPos.y;
        }
    };
    ScrollSection.prototype.calculateVerticalScrollSize = function (scrollLength) {
        var scrollSize = scrollLength * scrollLength / this.sectionProperties.documentHeight;
        if (scrollSize > this.sectionProperties.minimumScrollSize) {
            return Math.round(scrollSize);
        }
        else {
            return this.sectionProperties.minimumScrollSize;
        }
    };
    ScrollSection.prototype.getVerticalScrollProperties = function () {
        var result = {};
        result.scrollLength = this.getVerticalScrollLength(); // The length of the railway that the scroll bar moves on up & down.
        result.scrollSize = this.calculateVerticalScrollSize(result.scrollLength);
        result.scrollableLength = result.scrollLength - result.scrollSize;
        result.scrollableHeight = this.sectionProperties.documentHeight - this.size[1];
        result.ratio = result.scrollableHeight / result.scrollableLength; // 1px scrolling = xpx document height.
        result.startY = Math.round(this.documentTopLeft[1] / result.ratio + this.sectionProperties.scrollBarThickness * 0.5 + this.sectionProperties.yOffset);
        this.sectionProperties.documentTopMax = result.scrollableHeight; // When documentTopLeft[1] value is equal to this value, it means whole document is visible.
        return result;
    };
    ScrollSection.prototype.getHorizontalScrollLength = function () {
        if (this.map._docLayer._docType !== 'spreadsheet') {
            return this.size[0] - this.sectionProperties.horizontalScrollRightOffset;
        }
        else {
            var splitPanesContext = this.map.getSplitPanesContext();
            var splitPos = splitPanesContext.getSplitPos().clone();
            splitPos.x = Math.round(splitPos.x * this.dpiScale);
            this.sectionProperties.xOffset = splitPos.x;
            return this.size[0] - splitPos.x - this.sectionProperties.horizontalScrollRightOffset;
        }
    };
    ScrollSection.prototype.calculateHorizontalScrollSize = function (scrollLength) {
        var scrollSize = scrollLength * scrollLength / this.sectionProperties.documentWidth;
        if (scrollSize > this.sectionProperties.minimumScrollSize) {
            return Math.round(scrollSize);
        }
        else {
            return this.sectionProperties.minimumScrollSize;
        }
    };
    ScrollSection.prototype.getHorizontalScrollProperties = function () {
        var result = {};
        result.scrollLength = this.getHorizontalScrollLength(); // The length of the railway that the scroll bar moves on up & down.
        result.scrollSize = this.calculateHorizontalScrollSize(result.scrollLength); // Width of the scroll bar.
        result.scrollableLength = result.scrollLength - result.scrollSize;
        result.scrollableWidth = this.sectionProperties.documentWidth - this.size[0];
        result.ratio = result.scrollableWidth / result.scrollableLength;
        result.startX = Math.round(this.documentTopLeft[0] / result.ratio + this.sectionProperties.scrollBarThickness * 0.5 + this.sectionProperties.xOffset);
        this.sectionProperties.documentRightMax = result.scrollableWidth;
        return result;
    };
    ScrollSection.prototype.onUpdateScrollOffset = function () {
        if (this.map._docLayer._docType === 'spreadsheet')
            this.map._docLayer.refreshViewData();
    };
    ScrollSection.prototype.DrawVerticalScrollBarMobile = function () {
        var scrollProps = this.getVerticalScrollProperties();
        if (this.sectionProperties.animatingVerticalScrollBar)
            this.context.globalAlpha = this.sectionProperties.currentAlpha;
        else
            this.context.globalAlpha = this.sectionProperties.clickScrollVertical ? this.sectionProperties.alphaWhenBeingUsed : this.sectionProperties.alphaWhenVisible;
        this.context.strokeStyle = '#7E8182';
        this.context.fillStyle = 'white';
        var circleStartY = scrollProps.startY + this.sectionProperties.circleSliderRadius;
        var circleStartX = this.size[0] - this.sectionProperties.circleSliderRadius * 0.5;
        this.context.beginPath();
        this.context.arc(circleStartX, circleStartY, this.sectionProperties.circleSliderRadius, 0, Math.PI * 2, true);
        this.context.fill();
        this.context.stroke();
        this.context.fillStyle = '#7E8182';
        this.context.beginPath();
        var x = circleStartX - this.sectionProperties.arrowCornerLength * 0.5;
        var y = circleStartY - 5 * this.sectionProperties.roundedDpi;
        this.context.moveTo(x, y);
        x += this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        x -= this.sectionProperties.arrowCornerLength * 0.5;
        y -= Math.sin(Math.PI / 3) * this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        x -= this.sectionProperties.arrowCornerLength * 0.5;
        y += Math.sin(Math.PI / 3) * this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        this.context.fill();
        x = circleStartX - this.sectionProperties.arrowCornerLength * 0.5;
        y = circleStartY + 5 * this.sectionProperties.roundedDpi;
        this.context.moveTo(x, y);
        x += this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        x -= this.sectionProperties.arrowCornerLength * 0.5;
        y += Math.sin(Math.PI / 3) * this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        x -= this.sectionProperties.arrowCornerLength * 0.5;
        y -= Math.sin(Math.PI / 3) * this.sectionProperties.arrowCornerLength;
        this.context.lineTo(x, y);
        this.context.fill();
        this.context.globalAlpha = 1.0;
    };
    ScrollSection.prototype.drawVerticalScrollBar = function () {
        var scrollProps = this.getVerticalScrollProperties();
        if (this.sectionProperties.animatingVerticalScrollBar)
            this.context.globalAlpha = this.sectionProperties.currentAlpha;
        else
            this.context.globalAlpha = this.sectionProperties.clickScrollVertical ? this.sectionProperties.alphaWhenBeingUsed : this.sectionProperties.alphaWhenVisible;
        this.context.fillStyle = '#7E8182';
        var startX = this.size[0] - this.sectionProperties.scrollBarThickness - this.sectionProperties.edgeOffset;
        this.context.fillRect(startX + this.sectionProperties.roundedDpi, scrollProps.startY - this.sectionProperties.roundedDpi, this.sectionProperties.roundedDpi * 4, this.sectionProperties.roundedDpi);
        this.context.fillRect(startX, scrollProps.startY, this.sectionProperties.scrollBarThickness, scrollProps.scrollSize - this.sectionProperties.scrollBarThickness);
        this.context.fillRect(startX + this.sectionProperties.roundedDpi, scrollProps.startY + scrollProps.scrollSize - this.sectionProperties.scrollBarThickness, this.sectionProperties.roundedDpi * 4, this.sectionProperties.roundedDpi);
        this.context.globalAlpha = 1.0;
    };
    ScrollSection.prototype.drawHorizontalScrollBar = function () {
        var scrollProps = this.getHorizontalScrollProperties();
        if (this.sectionProperties.animatingHorizontalScrollBar)
            this.context.globalAlpha = this.sectionProperties.currentAlpha;
        else
            this.context.globalAlpha = this.sectionProperties.clickScrollHorizontal ? this.sectionProperties.alphaWhenBeingUsed : this.sectionProperties.alphaWhenVisible;
        this.context.fillStyle = '#7E8182';
        var startY = this.size[1] - this.sectionProperties.scrollBarThickness - this.sectionProperties.edgeOffset;
        this.context.fillRect(scrollProps.startX - this.sectionProperties.roundedDpi, startY + this.sectionProperties.roundedDpi, this.sectionProperties.roundedDpi, this.sectionProperties.roundedDpi * 4);
        this.context.fillRect(scrollProps.startX, startY, scrollProps.scrollSize - this.sectionProperties.scrollBarThickness, this.sectionProperties.scrollBarThickness);
        this.context.fillRect(scrollProps.startX + scrollProps.scrollSize - this.sectionProperties.scrollBarThickness, startY + this.sectionProperties.roundedDpi, this.sectionProperties.roundedDpi, this.sectionProperties.roundedDpi * 4);
        this.context.globalAlpha = 1.0;
    };
    ScrollSection.prototype.onDraw = function (frameCount, elapsedTime) {
        if (this.isAnimating && frameCount >= 0)
            this.sectionProperties.currentAlpha = Math.max((1 - (elapsedTime / 2000)) * this.sectionProperties.alphaWhenVisible, 0.1);
        if ((this.sectionProperties.drawVerticalScrollBar || this.sectionProperties.animatingVerticalScrollBar) && this.documentTopLeft[1] >= 0) {
            if (window.mode.isMobile())
                this.DrawVerticalScrollBarMobile();
            else
                this.drawVerticalScrollBar();
        }
        if ((this.sectionProperties.drawHorizontalScrollBar || this.sectionProperties.animatingHorizontalScrollBar) && this.documentTopLeft[0] >= 0) {
            this.drawHorizontalScrollBar();
        }
    };
    ScrollSection.prototype.onAnimationEnded = function (frameCount, elapsedTime) {
        this.sectionProperties.animatingVerticalScrollBar = false;
        this.sectionProperties.animatingHorizontalScrollBar = false;
    };
    ScrollSection.prototype.fadeOutHorizontalScrollBar = function () {
        if (this.isAnimating) {
            this.resetAnimation();
            this.sectionProperties.animatingHorizontalScrollBar = true;
        }
        else {
            var options = {
                duration: 2000
            };
            this.sectionProperties.animatingHorizontalScrollBar = this.startAnimating(options);
        }
    };
    ScrollSection.prototype.fadeOutVerticalScrollBar = function () {
        if (this.isAnimating) {
            this.resetAnimation();
            this.sectionProperties.animatingVerticalScrollBar = true;
        }
        else {
            var options = {
                duration: 2000
            };
            this.sectionProperties.animatingVerticalScrollBar = this.startAnimating(options);
        }
    };
    ScrollSection.prototype.hideVerticalScrollBar = function () {
        if (this.sectionProperties.mouseIsOnVerticalScrollBar) {
            this.sectionProperties.drawVerticalScrollBar = false;
            this.sectionProperties.mouseIsOnVerticalScrollBar = false;
            this.sectionProperties.mapPane.style.cursor = this.sectionProperties.defaultCursorStyle;
            this.fadeOutVerticalScrollBar();
            // just in case if we have blinking cursor visible
            // we need to change cursor from default style
            if (this.map._docLayer._cursorMarker)
                this.map._docLayer._cursorMarker.setMouseCursor();
        }
    };
    ScrollSection.prototype.showVerticalScrollBar = function () {
        if (this.isAnimating && this.sectionProperties.animatingVerticalScrollBar)
            this.containerObject.stopAnimating();
        if (!this.sectionProperties.mouseIsOnVerticalScrollBar) {
            this.sectionProperties.drawVerticalScrollBar = true;
            this.sectionProperties.mouseIsOnVerticalScrollBar = true;
            this.sectionProperties.mapPane.style.cursor = 'pointer';
            this.containerObject.requestReDraw();
        }
    };
    ScrollSection.prototype.hideHorizontalScrollBar = function () {
        if (this.sectionProperties.mouseIsOnHorizontalScrollBar) {
            this.sectionProperties.drawHorizontalScrollBar = false;
            this.sectionProperties.mouseIsOnHorizontalScrollBar = false;
            this.sectionProperties.mapPane.style.cursor = this.sectionProperties.defaultCursorStyle;
            this.fadeOutHorizontalScrollBar();
            // just in case if we have blinking cursor visible
            // we need to change cursor from default style
            if (this.map._docLayer._cursorMarker)
                this.map._docLayer._cursorMarker.setMouseCursor();
        }
    };
    ScrollSection.prototype.showHorizontalScrollBar = function () {
        if (this.isAnimating && this.sectionProperties.animatingHorizontalScrollBar)
            this.containerObject.stopAnimating();
        if (!this.sectionProperties.mouseIsOnHorizontalScrollBar) {
            this.sectionProperties.drawHorizontalScrollBar = true;
            this.sectionProperties.mouseIsOnHorizontalScrollBar = true;
            this.sectionProperties.mapPane.style.cursor = 'pointer';
            this.containerObject.requestReDraw();
        }
    };
    ScrollSection.prototype.isMouseOnScrollBar = function (point) {
        if (this.documentTopLeft[1] >= 0) {
            if (point[0] >= this.size[0] - this.sectionProperties.usableThickness) {
                if (point[1] > this.sectionProperties.yOffset) {
                    this.showVerticalScrollBar();
                }
                else {
                    this.hideVerticalScrollBar();
                }
            }
            else {
                this.hideVerticalScrollBar();
            }
        }
        if (this.documentTopLeft[0] >= 0) {
            if (point[1] >= this.size[1] - this.sectionProperties.usableThickness) {
                if (point[0] <= this.size[0] - this.sectionProperties.horizontalScrollRightOffset && point[0] >= this.sectionProperties.xOffset) {
                    this.showHorizontalScrollBar();
                }
                else {
                    this.hideHorizontalScrollBar();
                }
            }
            else {
                this.hideHorizontalScrollBar();
            }
        }
    };
    ScrollSection.prototype.onMouseLeave = function () {
        this.hideVerticalScrollBar();
        this.hideHorizontalScrollBar();
    };
    ScrollSection.prototype.scrollVerticalWithOffset = function (offset) {
        if (this.documentTopLeft[1] + offset <= 0) { // We shouldn't scroll document to a negative y value.
            if (this.documentTopLeft[1] > 0)
                this.map.scrollTop(0, {});
        }
        else if (this.documentTopLeft[1] + offset >= this.sectionProperties.documentTopMax) // We should stop at the bottom of the document.
            this.map.scrollTop(this.sectionProperties.documentTopMax / this.dpiScale, {});
        else // Humph, everything is normal.
            this.map.scroll(0, offset / this.dpiScale, {});
        this.onUpdateScrollOffset();
    };
    ScrollSection.prototype.scrollHorizontalWithOffset = function (offset) {
        if (this.documentTopLeft[0] + offset <= 0) { // We shouldn't scroll document to a negative x value.
            if (this.documentTopLeft[0] > 0)
                this.map.scrollLeft(0, {});
        }
        else if (this.documentTopLeft[0] + offset >= this.sectionProperties.documentRightMax) // We should stop at the right edge of the document.
            this.map.scrollLeft(this.sectionProperties.documentRightMax / this.dpiScale, {});
        else // Humph, everything is normal.
            this.map.scroll(offset / this.dpiScale, 0, {});
        this.onUpdateScrollOffset();
    };
    ScrollSection.prototype.onMouseMove = function (position, dragDistance, e) {
        if (this.sectionProperties.clickScrollVertical && this.containerObject.draggingSomething) {
            if (!this.sectionProperties.previousDragDistance) {
                this.sectionProperties.previousDragDistance = [0, 0];
            }
            this.showVerticalScrollBar();
            var scrollProps = this.getVerticalScrollProperties();
            var diffY = dragDistance[1] - this.sectionProperties.previousDragDistance[1];
            var actualDistance = scrollProps.ratio * diffY;
            this.scrollVerticalWithOffset(actualDistance);
            this.sectionProperties.previousDragDistance[1] = dragDistance[1];
            e.stopPropagation(); // Don't propagate to map.
            this.stopPropagating(); // Don't propagate to bound sections.
        }
        else if (this.sectionProperties.clickScrollHorizontal && this.containerObject.draggingSomething) {
            if (!this.sectionProperties.previousDragDistance) {
                this.sectionProperties.previousDragDistance = [0, 0];
            }
            this.showHorizontalScrollBar();
            var scrollProps = this.getHorizontalScrollProperties();
            var diffX = dragDistance[0] - this.sectionProperties.previousDragDistance[0];
            var percentage = diffX / scrollProps.scrollLength;
            var actualDistance = this.sectionProperties.documentWidth * percentage;
            this.scrollHorizontalWithOffset(actualDistance);
            this.sectionProperties.previousDragDistance[0] = dragDistance[0];
            e.stopPropagation(); // Don't propagate to map.
            this.stopPropagating(); // Don't propagate to bound sections.
        }
        else {
            this.isMouseOnScrollBar(position);
        }
    };
    ScrollSection.prototype.onMouseDown = function (point, e) {
        this.onMouseMove(point, null, e);
        this.isMouseOnScrollBar(point);
        if (this.documentTopLeft[1] >= 0) {
            if (point[0] >= this.size[0] - this.sectionProperties.usableThickness) {
                if (point[1] > this.sectionProperties.yOffset) {
                    this.sectionProperties.clickScrollVertical = true;
                    this.map.scrollingIsHandled = true;
                    e.stopPropagation(); // Don't propagate to map.
                    this.stopPropagating(); // Don't propagate to bound sections.
                }
                else {
                    this.sectionProperties.clickScrollVertical = false;
                }
            }
            else {
                this.sectionProperties.clickScrollVertical = false;
            }
        }
        if (this.documentTopLeft[0] >= 0) {
            if (point[1] >= this.size[1] - this.sectionProperties.usableThickness) {
                if (point[0] >= this.sectionProperties.xOffset && point[0] <= this.size[0] - this.sectionProperties.horizontalScrollRightOffset) {
                    this.sectionProperties.clickScrollHorizontal = true;
                    this.map.scrollingIsHandled = true;
                    e.stopPropagation(); // Don't propagate to map.
                    this.stopPropagating(); // Don't propagate to bound sections.
                }
                else {
                    this.sectionProperties.clickScrollHorizontal = false;
                }
            }
            else {
                this.sectionProperties.clickScrollHorizontal = false;
            }
        }
    };
    ScrollSection.prototype.onMouseUp = function (point, e) {
        this.map.scrollingIsHandled = false;
        if (this.sectionProperties.clickScrollVertical) {
            e.stopPropagation(); // Don't propagate to map.
            this.stopPropagating(); // Don't propagate to bound sections.
            this.sectionProperties.clickScrollVertical = false;
        }
        if (this.sectionProperties.clickScrollHorizontal) {
            e.stopPropagation(); // Don't propagate to map.
            this.stopPropagating(); // Don't propagate to bound sections.
            this.sectionProperties.clickScrollHorizontal = false;
        }
        this.sectionProperties.previousDragDistance = null;
        this.onMouseMove(point, null, e);
    };
    ScrollSection.prototype.performVerticalScroll = function (delta) {
        this.scrollVerticalWithOffset(delta);
        if (!this.sectionProperties.drawVerticalScrollBar) {
            if (this.isAnimating) {
                this.resetAnimation();
                this.sectionProperties.animatingVerticalScrollBar = true;
            }
            else
                this.fadeOutVerticalScrollBar();
        }
    };
    ScrollSection.prototype.performHorizontalScroll = function (delta) {
        this.scrollHorizontalWithOffset(delta);
        if (!this.sectionProperties.drawHorizontalScrollBar) {
            if (this.isAnimating) {
                this.resetAnimation();
                this.sectionProperties.animatingHorizontalScrollBar = true;
            }
            else
                this.fadeOutHorizontalScrollBar();
        }
    };
    ScrollSection.prototype.onMouseWheel = function (point, delta, e) {
        if (e.ctrlKey)
            return;
        if (Math.abs(delta[1]) > Math.abs(delta[0])) {
            if (!e.shiftKey)
                this.performVerticalScroll(delta[1]);
            else
                this.performHorizontalScroll(delta[1]);
        }
        else {
            this.performHorizontalScroll(delta[0]);
        }
    };
    ScrollSection.prototype.onMouseEnter = function () { };
    ScrollSection.prototype.onClick = function () { };
    ScrollSection.prototype.onDoubleClick = function () { };
    ScrollSection.prototype.onContextMenu = function () { };
    ScrollSection.prototype.onLongPress = function () { };
    ScrollSection.prototype.onMultiTouchStart = function () { };
    ScrollSection.prototype.onMultiTouchMove = function () { };
    ScrollSection.prototype.onMultiTouchEnd = function () { };
    ScrollSection.prototype.onResize = function () { };
    ScrollSection.prototype.onNewDocumentTopLeft = function () { };
    return ScrollSection;
}());
L.getNewScrollSection = function () {
    return new ScrollSection();
};
